/***
 * 
 * Copyright (C) 2009 Alessandro La Rosa
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * Contact: alessandro.larosa@gmail.com
 *
 * Author: Alessandro La Rosa
 */

function Slider(_id, _defaultValue, _minValue, _maxValue, _callback)
{
	this.id = _id;
	this.sliderInput = document.getElementById(_id);
	
	this.slider = null;
	this.sliderHandle = null;
	this.sliderValueElement = null;
	
	this.value = (_defaultValue ? _defaultValue : 0);
	this.minValue = (_minValue ? _minValue : 0);
	this.maxValue = (_maxValue ? _maxValue : 0);
	
	this.callback = _callback;
	
	(this.sliderInput) ? this.init() : false;
}
Slider.prototype.init = function()
{
	//hide the text input
	this.sliderInput.style.display = 'none';
	
	//create the slider <ul> element
	this.slider = document.createElement('ul');
	this.slider.setAttribute('class', 'slider');
	
	//create the slider <li> element
	var li = document.createElement('li');
	li.setAttribute('class', 'slider-item');
	
	//create the <a> element
	var aElement = document.createElement('a');
	aElement.href = "#";
	
	//create the <span> element (the slider handle)
	this.sliderHandle = document.createElement('span');
	
	//create the slider value <li> element
	var valueLi = document.createElement('li');
	valueLi.setAttribute('class', 'slider-value');
	
	//create the value element
	this.sliderValueElement = document.createElement('p');
	this.sliderValueElement.innerHTML = this.value;
	
	//appends the elements
	aElement.appendChild(this.sliderHandle);
	li.appendChild(aElement);
	this.slider.appendChild(li);
	
	valueLi.appendChild(this.sliderValueElement);
	this.slider.appendChild(valueLi);
	
	//replace the input with the slider element
	this.sliderInput.parentNode.insertBefore(this.slider, this.sliderInput);
	
	var self = this;
	
	aElement.onkeydown = function(e)
	{
		self.handleKeyDown(e);
	}
	
	this.sliderHandle.onclick = function(e)
	{
		self.handleClicked(e);
	}
	
	this.setValue(this.value);
}
Slider.prototype.handleKeyDown = function(e)
{
	var key = e.keyCode;
	
	var newValue = this.value;
	
	if(key == 37 && this.value > this.minValue)
		newValue--;
	else if(key == 39 && this.value < this.maxValue)
		newValue++;
	
	if(newValue != this.value)
		this.setValue(newValue);
}
Slider.prototype.handleClicked = function(e)
{
	var mouseX = e.clientX;

	var relativeX = mouseX - this.sliderHandle.offsetLeft;
	
	var newValue = this.minValue + Math.round((this.maxValue - this.minValue) * relativeX / this.sliderHandle.offsetWidth);
	
	this.setValue(newValue);
}
	
Slider.prototype.update = function()
{
	var positionX = 100 * (this.value - this.minValue) / (this.maxValue - this.minValue);

	this.sliderHandle.style.backgroundPosition = positionX + '% center';
	
	this.sliderInput.value = this.value;
}
Slider.prototype.setValue = function(_value)
{
	this.sliderValueElement.innerHTML = _value;
	
	this.value = _value;
	
	this.update();
	
	if(this.callback)
	{
		this.callback(this);
	}
}
